<?php

use Fpdf\Fpdf;
use PhpOffice\PhpPresentation\PhpPresentation;
use PhpOffice\PhpPresentation\Shape\Drawing;

error_reporting(E_ALL);
ini_set('display_errors', 1); // Enable error display
ini_set('log_errors', 1);    // Enable error logging
ini_set('error_log', 'error.log'); // Log file path

require 'vendor/autoload.php'; // Include Composer autoloader

header('Content-Type: application/json'); // Ensure JSON output

function curl_get_contents($url, $retries = 3, $timeout = 30)
{
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
    $data = curl_exec($ch);
    if (curl_errno($ch) && $retries > 0) {
        curl_close($ch);
        return curl_get_contents($url, $retries - 1, $timeout);
    }
    curl_close($ch);
    return $data;
}

function multi_curl_get_contents($urls, $retries = 3, $timeout = 30)
{
    $mh = curl_multi_init();
    $handles = [];
    $responses = [];

    foreach ($urls as $i => $url) {
        $handles[$i] = curl_init($url);
        curl_setopt($handles[$i], CURLOPT_RETURNTRANSFER, true);
        curl_setopt($handles[$i], CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($handles[$i], CURLOPT_TIMEOUT, $timeout);
        curl_multi_add_handle($mh, $handles[$i]);
    }

    $active = null;
    do {
        $mrc = curl_multi_exec($mh, $active);
    } while ($mrc == CURLM_CALL_MULTI_PERFORM);

    while ($active && $mrc == CURLM_OK) {
        if (curl_multi_select($mh) != -1) {
            do {
                $mrc = curl_multi_exec($mh, $active);
            } while ($mrc == CURLM_CALL_MULTI_PERFORM);
        }
    }

    foreach ($handles as $i => $handle) {
        $responses[$i] = curl_multi_getcontent($handle);
        curl_multi_remove_handle($mh, $handle);
    }

    curl_multi_close($mh);
    return $responses;
}

$directoryPath = '/home/handsyeo/public_html/downloads/';
try {
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        $rawInput = file_get_contents('php://input');
        $input = json_decode($rawInput, true);

        // Validate input
        if (!isset($input['url']) || !filter_var($input['url'], FILTER_VALIDATE_URL)) {
            echo json_encode(['success' => false, 'message' => 'Invalid URL provided.']);
            exit;
        }
        if (!isset($input['type']) || !in_array($input['type'], ['pdf', 'pptx', 'zip'])) {
            echo json_encode(['success' => false, 'message' => 'Invalid download type.']);
            exit;
        }
        if (!isset($input['quality']) || !in_array($input['quality'], ['high', 'medium', 'low'])) {
            echo json_encode(['success' => false, 'message' => 'Invalid quality.']);
            exit;
        }

        $slideshareUrl = $input['url'];
        // $slidesName = preg_replace('/[^a-zA-Z0-9\-]/', '', basename(parse_url($slideshareUrl, PHP_URL_PATH))) ?: 'slideshare';
        $slidesName = preg_replace('/[^a-zA-Z0-9\-]/', '', basename(parse_url($slideshareUrl, PHP_URL_PATH))) ?: 'slideshare';
        $uniqueId = uniqid(); // Add a unique ID to the filename
        $outputType = $input['type'];
        $quality = $input['quality'];
        
        $outputPath = $directoryPath . $slidesName . '_' . $uniqueId . '.' . $outputType;
        // $outputPath = $directoryPath . $slidesName . '.' . $outputType;
        if (file_exists($outputPath)) {
            // If the file already exists, return its path
            echo json_encode([
                'success' => true,
                'message' => 'File already exists.',
                'download_link' => "https://slidesharedl.com/downloads/" . $slidesName . '_' . $uniqueId . '.' . $outputType
            ]);
            exit;
        }

        $html = curl_get_contents($slideshareUrl);
        if ($html === false) {
            echo json_encode(['success' => false, 'message' => 'Failed to fetch the SlideShare page.']);
            exit;
        }

        preg_match_all('/srcSet="([^"]+)"/', $html, $matches);
        if (empty($matches[1])) {
            echo json_encode(['success' => false, 'message' => 'No images found on the SlideShare page.']);
            exit;
        }

        $imageUrls = [];
        foreach ($matches[1] as $srcSet) {
            foreach (explode(',', $srcSet) as $url) {
                $urlParts = explode(' ', trim($url));
                $imageUrl = $urlParts[0];
                if (strpos($imageUrl, 'image.slidesharecdn.com') !== false && preg_match('/-638\.jpg$/', $imageUrl)) {
                    $imageUrls[] = $imageUrl;
                }
            }
        }

        $uniqueUrls = array_unique($imageUrls);

        if ($outputType === 'pdf') {
            $pdf = new FPDF();
            $tempFiles = [];
            $imageContents = multi_curl_get_contents($uniqueUrls);

            foreach ($imageContents as $index => $imageContent) {
                $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
                file_put_contents($tempImagePath, $imageContent);
                $imageInfo = @getimagesize($tempImagePath);

                if ($imageInfo === false) {
                    unlink($tempImagePath);
                    continue;
                }

                $width = $imageInfo[0] * 0.264583;
                $height = $imageInfo[1] * 0.264583;

                $pdf->AddPage($width > $height ? 'L' : 'P', [$width, $height]);
                $x = ($pdf->GetPageWidth() - $width) / 2;
                $y = ($pdf->GetPageHeight() - $height) / 2;

                $pdf->Image($tempImagePath, $x, $y, $width, $height);
                $tempFiles[] = $tempImagePath;
            }

            $pdf->Output($outputPath, 'F');

            // Clean up temporary files
            foreach ($tempFiles as $tempFile) {
                if (file_exists($tempFile)) {
                    unlink($tempFile);
                }
            }

            // Ensure the file exists before returning the response
            if (file_exists($outputPath)) {
                echo json_encode([
                    'success' => true,
                    'message' => 'PDF generated successfully.',
                    'download_link' => "https://slidesharedl.com/downloads/" . $slidesName . '_' . $uniqueId . '.' . $outputType
                ]);
            } else {
                echo json_encode(['success' => false, 'message' => 'Failed to generate PDF file.']);
            }
            exit;
        } elseif ($outputType === 'zip') {
            $zip = new ZipArchive();

            if ($zip->open($outputPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === true) {
                $imageContents = multi_curl_get_contents($uniqueUrls);

                foreach ($imageContents as $index => $imageContent) {
                    if ($imageContent !== false) {
                        $fileName = basename($uniqueUrls[$index]);
                        $zip->addFromString($fileName, $imageContent);
                        $zip->setCompressionIndex($index, ZipArchive::CM_DEFLATE);
                    }
                }
                $zip->close();
            } else {
                echo json_encode(['success' => false, 'message' => 'Failed to create ZIP archive.']);
                exit;
            }

            // Ensure the file exists before returning the response
            if (file_exists($outputPath)) {
                echo json_encode([
                    'success' => true,
                    'message' => 'ZIP generated successfully.',
                    'download_link' => "https://slidesharedl.com/downloads/" . $slidesName . '_' . $uniqueId . '.' . $outputType
                ]);
            } else {
                echo json_encode(['success' => false, 'message' => 'Failed to generate ZIP file.']);
            }
            exit;
        }elseif ($outputType === 'pptx') {
            $presentation = new PhpPresentation();
            $presentation->removeSlideByIndex(0);
            $tempFiles = []; // Array to store temporary file paths
            $imageContents = multi_curl_get_contents($uniqueUrls);
        
            foreach ($imageContents as $index => $imageContent) {
                if (@imagecreatefromstring($imageContent) === false) {
                    continue; // Skip invalid images
                }
        
                $tempImagePath = $directoryPath . 'temp_image_' . uniqid() . '.jpg';
                file_put_contents($tempImagePath, $imageContent);
                $tempFiles[] = $tempImagePath; // Add temporary file path to the array
        
                $imageInfo = @getimagesize($tempImagePath);
                if ($imageInfo === false) {
                    unlink($tempImagePath); // Delete invalid temporary file
                    continue;
                }
        
                $imageWidth = $imageInfo[0];
                $imageHeight = $imageInfo[1];
                $slideWidth = 960;
                $slideHeight = 720;
        
                $scaleFactor = min($slideWidth / $imageWidth, $slideHeight / $imageHeight);
                $newWidth = (int) round($imageWidth * $scaleFactor);
                $newHeight = (int) round($imageHeight * $scaleFactor);
                $offsetX = (int) round(($slideWidth - $newWidth) / 2);
                $offsetY = (int) round(($slideHeight - $newHeight) / 2);
        
                $currentSlide = $presentation->createSlide();
                $shape = $currentSlide->createDrawingShape();
                $shape->setName('Image ' . $index)
                    ->setDescription('Slide ' . ($index + 1))
                    ->setPath($tempImagePath)
                    ->setWidth($newWidth)
                    ->setHeight($newHeight)
                    ->setOffsetX($offsetX)
                    ->setOffsetY($offsetY);
        
                $shape->getShadow()->setVisible(true)->setDirection(45)->setDistance(10);
            }
        
            // error_log("Generating PPT file: " . $outputPath);
            $oWriterPPTX = new PhpOffice\PhpPresentation\Writer\PowerPoint2007($presentation);
            $oWriterPPTX->save($outputPath);
            // error_log("PPT file generated: " . $outputPath);
        
            // Clean up temporary files
            foreach ($tempFiles as $tempFile) {
                if (file_exists($tempFile)) {
                    unlink($tempFile); // Delete temporary file
                    // error_log("Deleted temporary file: " . $tempFile);
                }
            }
        
            // Ensure the file is fully written
            $file = fopen($outputPath, 'r');
            if (flock($file, LOCK_SH)) { // Acquire a shared lock
                flock($file, LOCK_UN); // Release the lock
                fclose($file);
        
                if (file_exists($outputPath)) {
                    error_log("PPT file exists and is accessible: " . $outputPath);
                    echo json_encode([
                        'success' => true,
                        'message' => 'PPT generated successfully.',
                        'download_link' => "https://slidesharedl.com/downloads/" . $slidesName . '_' . $uniqueId . '.' . $outputType
                    ]);
                } else {
                    error_log("PPT file does not exist: " . $outputPath);
                    echo json_encode(['success' => false, 'message' => 'Failed to generate PPT file.']);
                }
            } else {
                echo json_encode(['success' => false, 'message' => 'Failed to lock file for reading.']);
            }
            exit;
        } else {
            echo json_encode(['success' => false, 'message' => 'Invalid output type specified.']);
        }
    } else {
        echo json_encode(['success' => false, 'message' => 'Invalid request method.']);
    }
} catch (Exception $e) {
    echo json_encode(['success' => false, 'message' => 'An error occurred: ' . $e->getMessage()]);
}
