<?php

error_reporting(E_ALL);
ini_set('display_errors', 0);
ini_set('log_errors', 1);
ini_set('error_log', 'error.log');

require 'vendor/autoload.php';

use GuzzleHttp\Client;
use GuzzleHttp\Promise;

header('Content-Type: application/json');

function extractImageUrls($html, $quality) {
    $dom = new DOMDocument();
    @$dom->loadHTML($html);
    $xpath = new DOMXPath($dom);
    $imageUrls = [];
    $nodes = $xpath->query("//img[contains(@srcset, 'image.slidesharecdn.com')]");
    foreach ($nodes as $node) {
        $srcset = $node->getAttribute('srcset');
        $urls = explode(',', $srcset);
        foreach ($urls as $url) {
            $parts = explode(' ', trim($url));
            if (preg_match('/-' . $quality . '\.jpg$/', $parts[0])) {
                $imageUrls[] = $parts[0];
            }
        }
    }
    return array_unique($imageUrls);
}

try {
    if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
        throw new Exception('Invalid request method.');
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (!isset($input['url']) || !filter_var($input['url'], FILTER_VALIDATE_URL)) {
        throw new Exception('Invalid URL provided.');
    }

    $slideshareUrl = $input['url'];
    // Determine quality: Low = 320, Medium = 638, High = 2048.
    $quality = $input['quality'] === 'Low' ? 320 : ($input['quality'] === 'Medium' ? 638 : 2048);

    // Increase timeout values to allow slower image loads
    $client = new Client([
        'timeout' => 20,
        'connect_timeout' => 20,
    ]);
    $response = $client->get($slideshareUrl);
    $html = $response->getBody()->getContents();

    $imageUrls = extractImageUrls($html, $quality);
    if (empty($imageUrls)) {
        throw new Exception('No images found on the SlideShare page.');
    }

    $slidesName = preg_replace('/[^a-zA-Z0-9\-]/', '', basename(parse_url($slideshareUrl, PHP_URL_PATH)));

    // Create promises for all image URLs
    $promises = [];
    foreach ($imageUrls as $url) {
        $promises[$url] = $client->getAsync($url);
    }

    // Wait for all the promises to settle
    $results = Promise\Utils::settle($promises)->wait();

    $successfulUrls = [];
    foreach ($results as $url => $result) {
        if ($result['state'] === 'fulfilled') {
            $successfulUrls[] = $url;
        } else {
            // If the image URL ends with "-396.jpg", try replacing it with "-987.jpg"
            if (strpos($url, '-396.jpg') !== false) {
                $fallbackUrl = str_replace('-396.jpg', '-987.jpg', $url);
                try {
                    $fallbackResponse = $client->get($fallbackUrl, [
                        'timeout' => 20,
                        'connect_timeout' => 20,
                    ]);
                    if ($fallbackResponse->getStatusCode() == 200) {
                        $successfulUrls[] = $fallbackUrl;
                    }
                } catch (Exception $fallbackEx) {
                    error_log("Fallback failed for URL $url: " . $fallbackEx->getMessage());
                }
            } else {
                // Log any other failures
                error_log("Failed to fetch URL $url.");
            }
        }
    }

    if (empty($successfulUrls)) {
        throw new Exception('No images could be fetched successfully.');
    }

    echo json_encode([
        'success'  => true,
        'slides'   => $successfulUrls,
        'filename' => $slidesName
    ]);

} catch (Exception $e) {
    echo json_encode([
        'success' => false,
        'message' => $e->getMessage()
    ]);
}
